Frequently Asked Questions
==========================

How to compile my own C++ code that uses Essentia?
--------------------------------------------------


Here is an example how to compile [standard_mfcc.cpp](https://github.com/MTG/essentia/blob/2.0.1/src/examples/standard_mfcc.cpp) example on Linux linking with a system-wide installation of Essentia (done by ```./waf install``) and all its dependencies. Modify to your needs. 

```
g++ -pipe -Wall -O2 -fPIC -I/usr/local/include/essentia/ -I/usr/local/include/essentia/scheduler/ -I/usr/local/include/essentia/streaming/  -I/usr/local/include/essentia/utils -I/usr/include/taglib -I/usr/local/include/gaia2 -I/usr/include/qt4 -I/usr/include/qt4/QtCore -D__STDC_CONSTANT_MACROS standard_mfcc.cpp -o standard_mfcc -L/usr/local/lib -lessentia -lfftw3 -lyaml -lavcodec -lavformat -lavutil -lsamplerate -ltag -lfftw3f -lQtCore -lgaia2
```

Alternatively, if you want to create and build your own examples, the easiest way is to add them to ```src/examples``` folder, modify ```src/examples/wscript``` file accordingly and use ```./waf configure --with-examples; ./waf``` to build them.

You can build your application using XCode (OSX) following [these steps](https://github.com/MTG/essentia/issues/58#issuecomment-38530548).


OSX static builds and templates (JUCE/VST and openFrameworks)
------------------------------------------------------------------------------------------------------------

Here you can find portable 32-bit static builds of the Essentia C++ library and its dependencies for OSX (thanks to Cárthach from GiantSteps) as well as templates for JUCE/VST and openFrameworks:

https://github.com/GiantSteps/Essentia-Libraries 



Linux static builds
-------------------

To create static build of the library and executable example extractors:

Prepare static build of Libav 0.8.16:
```
git clone git://git.libav.org/libav.git
cd libav
git checkout tags/v0.8.16
./configure --disable-doc --disable-ffmpeg --disable-avconv --disable-avplay --disable-avprobe --disable-avserver --disable-avdevice  --disable-swscale --disable-avfilter --disable-network --disable-indevs --disable-outdevs
make
sudo make install
``` 

Prepare static build of Taglib 1.9.1:
  
```
wget http://taglib.github.io/releases/taglib-1.9.1.tar.gz
tar xf taglib-1.9.1.tar.gz
cd taglib-1.9.1
cmake -DCMAKE_INSTALL_PREFIX=/usr/local -DCMAKE_RELEASE_TYPE=Release -DENABLE_STATIC=ON
make
sudo make install
```
Make sure you did not have libtag1-dev package installed via apt-get (if so, uninstall it).


Build Essentia:
```
./waf configure  --with-static-examples
./waf
sudo ./waf install
```

The static executables will be in the ```build/src/examples``` folder.


Cross-compiling for Windows on Linux
------------------------------------

Install mingw gcc/g++
```sudo apt-get install gcc-mingw32 ```

Build dependencies
```
cd packaging/win32_3rdparty
build_libav_nomuxers.sh
build_taglib.sh
build_fftw3.sh
build_libsamplerate.sh
build_yaml.sh
cd ../../
```

Build Essentia
```
./waf configre --with-examples --cross-compile-mingw32
./waf
```


How to know which other Algorithms an Algorithm uses?
-----------------------------------------------------

The most obvious answer is: by reading its code. However, it is also possible to generate such a list automatically. 

Running the python script ```src/examples/python/show_algo_dependencies.py``` will output a list of all intermediate Algorithms created within each Algorithm in Essentia. It utilizes the logging framework and watches for messages generated by AlgorithmFactory at the moment of running ```create()``` method for each internal algorithm.  

Note, that you cannot be sure this list of dependencies is 100% correct as the script simply instantiates each algorithm to test for its dependencies, but does not run the ```compute``` stage. It is up to developers conscience to keep instantiations in a correct place, and if an Algorithm is being created on the ```compute``` stage, it will be unnoticed.
